#ifndef DDPLOT_H
#define DDPLOT_H

#include <qcolor.h>
#include <qstring.h>

#include "strings.h"

//------- version and release of ddplot --------
const QString VERSION = "2.2";

#ifdef Q_WS_X11
  const QString PLATFORM = "X11/";
#elif defined(Q_WS_WIN)
  const QString PLATFORM = "Win/";
#elif defined(Q_WS_MACX)
  const QString PLATFORM = "Mac/";
#else
  const QString PLATFORM = "";
#endif

//----------------------------------------------


// global constants
const int UNDEFINED = 0;

const int END_OF_LIST = -1;
const int MAX_ATOMS = 5000;
const int MAX_PLOTS = 100;        // max. number of plots inserted simultaneously in the tabWidget
const int MAX_LAYERS = 10;        // max. layers for the assignment of the atomic names

const int MAX_NEIGHBORS = 100;
const double MAX_NEIGHBOR_DIST = 0.867;

const double PAN_RATIO = 0.01;    // multiplication constant for panning - PAN_RATIO*SIZEX(Y) is the step
const double PAN_NORMAL = 1;      // normal panning
const double PAN_FAST = 3;        // fast panning

const int LOAD_SETTINGS = 1;
const int SAVE_SETTINGS = 2;

const int PLOT_ATOM_LAYERS = 1;   // recognizing the atoms acoording to their layer
const int PLOT_ATOM_TYPES  = 2;   // recognizing the atoms according to their type

const int AUTO = -1;              // automatic scaling of the displacements

const int ON = 1;
const int OFF = 0;

const int SCREW = 1;              // plot screw component
const int EDGE = 2;               // plot edge component
const int MIXED = 3;              // mixed component given by a projection vector

const int UNRELAXED = 0;          // plot unrelaxed positions of atoms
const int RELAXED = 1;            // plot relaxed positions of atoms
const int COMPOSITE = 2;          // composite view - relaxed+unrelaxed configuration

// global variables

extern char *ATOMPOS_NAME[3];
extern int ATOM_DIAM[10];
extern int ATOM_THICKNESS[10];
extern QColor ATOM_FGCOLOR[10];
extern QColor ATOM_BGCOLOR[10];
extern TString ATOM_NAME[10];
extern double CAXES[3][3];
extern int DISP_COMPONENT;
extern double DSCALE_FACT;
extern int NUM_FGCOLORS;
extern int NUM_BGCOLORS;
extern int NUM_DIAM;
extern int NUM_THICKNESS;
extern double PROJ_VECTOR[3];
extern int ARROW_SHORTEST;
extern int ARROW_THICKNESS;
extern int PLOT_TYPE;
extern bool STARTUP_DEBUG;
extern int ATOMPOS;
extern double ATOMDISP_SCALE;
extern int WINWIDTH, WINHEIGHT;   // width and height of the application window [pixels]
extern bool PRINT_MAG_DISP;
extern double ZOOM_FACT;
extern bool ATOM_NUMBERS;
extern bool SHOW_DCENTER;

extern QString SETTINGS_FILE;
extern QString PLOT_FILE;

#endif
