#ifndef MATLIB_H
#define MATLIB_H

#include <iostream.h>
#include <memory.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include "control.h"


/*---------------------------------------------------------------------------------------------------------------------
  class TArray - DEKLARACE
---------------------------------------------------------------------------------------------------------------------*/

template <class T, int rank>
class TArray {
public:
  T *data;
  long items;           // pocet prvku struktury celkem
  int dim;              // dimenze pole
  int len[4];           // rozmery pole
  int estep[4];         // skoky, po kterych se muze struktura nafukovat
  bool auto_enlarge;    // povoleni automaticke zvetseni struktury po skocich estep

  TArray() { };                        // konstruktor bez alokace prostoru
  TArray(int len1, ...);
  TArray(TArray<T,rank> &t);
  ~TArray();

  void Allocate(int len1, ...);
  void Free();

  T &operator()(int i);
  T &operator()(int i, int j);
  T &operator()(int i, int j, int k);
  T &operator()(int i, int j, int k, int l);

  TArray<T,rank> &operator+(TArray<T,rank> &t);
  TArray<T,rank> &operator+=(TArray<T,rank> &t);
  TArray<T,rank> &operator-(TArray<T,rank> &t);
  TArray<T,rank> &operator-=(TArray<T,rank> &t);
  TArray<T,rank> &operator=(double value) { Fill(value); return *this; }
  TArray<T,rank> &operator=(TArray<T,rank> &t);
  TArray<T,rank> &operator*(double value);
  TArray<T,rank> &operator*(TArray<T,rank> &A2);
  TArray<T,rank> &operator/(double value);

  void EnlargeStep(int step1, ...);

  // vyplneni pole hodnotou value
  void Fill(double value);

  // zjisteni delky tenzoru v dir-tem smeru
  int Length(int dir) { return len[dir-1]; }

  // zjisteni dimenze pole
  int Dim(void) { return dim; }

  // zvetseni struktury podle skoku estep (auto_enlarge=true)
  void Resize(int i, int j, int k, int l);

  // prevod vektoru na matici a matice na vektor (pouze radkove matice (1,n))
  TArray<double,2> &ToMatrix();
  TArray<double,1> &ToVector();
};


typedef TArray<double,1> DVector;
typedef TArray<double,2> DMatrix;
typedef TArray<double,3> DArray3;
typedef TArray<double,4> DArray4;

// vypsani prvku tenzoru na obrazovku
ostream &operator<<(ostream &outp, DVector &A);
ostream &operator<<(ostream &outp, DMatrix &A);
ostream &operator<<(ostream &outp, DArray3 &A);
ostream &operator<<(ostream &outp, DArray4 &A);

// supremum - nejblizsi vyssi cele cislo:  sup(1.3)=2, sup(5.8)=2, sup(6)=6
int sup(double value);


/*---------------------------------------------------------------------------------------------------------------------
  class TArray - KONSTRUKTORY & DESTRUKTORY
---------------------------------------------------------------------------------------------------------------------*/

// konstruktor
template<class T, int rank>
TArray<T,rank>::TArray(int len1, ...)
{
  va_list args;
  int i;

  va_start(args,len1);
  dim = rank;
  len[0] = items = len1;
  for (i=1; i<rank; i++) {
    len[i] = va_arg(args,int);
    items *= len[i];
  }
  for (i=rank; i<4; i++)
    len[i] = 0;
  va_end(args);

  auto_enlarge = false;
  for (i=0; i<4; i++)
    estep[i] = 0;

  data = (T *) malloc(items*sizeof(T));
  Fill(0);
}


// copy kontruktor
template<class T, int rank>
TArray<T,rank>::TArray(TArray<T,rank> &t)
{
  int i;

  dim = t.dim;
  items = t.items;
  for (i=0; i<4; i++) {
    len[i] = t.len[i];
    estep[i] = t.estep[i];
  }

  data = (T *) malloc(t.items*sizeof(T));
  memcpy(data,t.data,t.items*sizeof(T));
}


// destruktor
template<class T, int rank>
TArray<T,rank>::~TArray()
{
  free(data);
  data = NULL;
}


// alokace struktury za behu programu
// - po deklaraci: TArray<T,rank> A;
// - pouziti:      A.Allocate(...);
template<class T, int rank>
void TArray<T,rank>::Allocate(int len1, ...)
{
  va_list args;
  int i;

  va_start(args,len1);
  dim = rank;
  len[0] = items = len1;
  for (i=1; i<rank; i++) {
    len[i] = va_arg(args,int);
    items *= len[i];
  }
  for (i=rank; i<4; i++)
    len[i] = 0;
  va_end(args);

  auto_enlarge = false;
  for (i=0; i<4; i++)
    estep[i] = 0;

  data = (T *) malloc(items*sizeof(T));
  Fill(0);
}


// uvolneni struktury za behu programu
template<class T, int rank>
void TArray<T,rank>::Free()
{
  free(data);
  data = NULL;
}


/*---------------------------------------------------------------------------------------------------------------------
  class TArray - OPERATORY
---------------------------------------------------------------------------------------------------------------------*/

// cteni/zapis do struktury
template<class T, int rank>
T &TArray<T,rank>::operator()(int i)
{
  if (Dim()!=1)
    ErrMessage("Incorrect number of indexes in reference to a matrix item.","TArray<T,rank>::operator()");

  if (i<1 || i>len[0]) {
    if (!auto_enlarge)
      ErrMessage("Attempt to read/write outside an array.","TArray<T,rank>::operator()");
    else
      Resize(i,0,0,0);
  }

  return data[i-1];
}


// cteni/zapis do struktury
template<class T, int rank>
T &TArray<T,rank>::operator()(int i, int j)
{
  if (Dim()!=2)
    ErrMessage("Incorrect number of indexes in reference to a matrix item.","TArray<T,rank>::operator()");

  if (i<1 || i>len[0] || j<1 || j>len[1]) {
    if (!auto_enlarge)
      ErrMessage("Attempt to read/write outside an array.","TArray<T,rank>::operator()");
    else
      Resize(i,j,0,0);
  }

  return data[len[1]*(i-1)+j-1];
}


// cteni/zapis do struktury
template<class T, int rank>
T &TArray<T,rank>::operator()(int i, int j, int k)
{
  if (Dim()!=3)
    ErrMessage("Incorrect number of indexes in reference to a matrix item.","TArray<T,rank>::operator()");

  if (i<1 || i>len[0] || j<1 || j>len[1] || k<1 || k>len[2]) {
    if (!auto_enlarge)
      ErrMessage("Attempt to read/write outside an array.","TArray<T,rank>::operator()");
    else
      Resize(i,j,k,0);
  }

  return data[len[0]*len[1]*(k-1)+len[1]*(i-1)+j-1];
}


// cteni/zapis do struktury
template<class T, int rank>
T &TArray<T,rank>::operator()(int i, int j, int k, int l)
{
  if (Dim()!=4)
    ErrMessage("Incorrect number of indexes in reference to a matrix item.","TArray<T,rank>::operator()");

  if (i<1 || i>len[0] || j<1 || j>len[1] || k<1 || k>len[2] || l<1 || l>len[3]) {
    if (!auto_enlarge)
      ErrMessage("Attempt to read/write outside an array.","TArray<T,rank>::operator()");
    else
      Resize(i,j,k,l);
  }

  return data[len[0]*len[1]*len[2]*(l-1)+len[0]*len[1]*(k-1)+len[1]*(i-1)+j-1];
}


// operator scitani matic - pokud maji tenzory stejnou velikost
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator+(TArray<T,rank> &t)
{
  char *fun="TArray<T,rank>::operator+";
  static TArray<T,rank> result = t;
  long int i;

  if (this->dim!=t.dim || this->items!=t.items)
    ErrMessage("Addition of dimensionally incompatible arrays.",fun);

  for (i=0; i<items; i++)
    result.data[i] = data[i]+t.data[i];

  return result;
}


// operator pricteni k vysledku - pokud maji tenzory stejnou velikost
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator+=(TArray<T,rank> &t)
{
  *this = *this+t;
  return *this;
}


// operator odecitani
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator-(TArray<T,rank> &t)
{
  char *fun="TArray<T,rank>::operator+";
  static TArray<T,rank> result = t;
  long int i;

  if (this->dim!=t.dim || this->items!=t.items)
    ErrMessage("Subtraction of dimensionally incompatible arrays.",fun);

  for (i=0; i<items; i++)
    result.data[i] = data[i]-t.data[i];

  return result;
}


// operator odecteni od vysledku - pokud maji tenzory stejnou velikost
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator-=(TArray<T,rank> &t)
{
  *this = *this-t;
  return *this;
}


// operator prirazeni
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator=(TArray<T,rank> &t)
{
  char *fun="TArray<T,rank>::operator=";
  long int i;

  if (this->dim!=t.dim || this->items!=t.items)
    ErrMessage("Operation with dimensionally incompatible arrays.",fun);

  for (i=0; i<items; i++)
    data[i] = t.data[i];

  return *this;
}


// operator nasobeni cislem zleva
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator*(double value)
{
  long int i;

  for (i=0; i<items; i++)
    data[i] *= value;
  return *this;
}


// operator nasobeni dvou tenzoru
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator*(TArray<T,rank> &A2)
{
  char *fun="TArray<T,rank>::operator*";
  static TArray<T,rank> A1 = *this;
  static TArray<T,rank> target;               // jen deklarace, bez alokace - ta se provede dole
  double sum;
  int new_len[4];
  int new_dim;
  int rmax1,cmax1,cmax2,d,dmax;
  int row,col,idx;

  // je nasobeni mozne?
  if (A1.Length(A1.dim)!=A2.Length(1))
    ErrMessage("Multiplication of dimensionally incompatible arrays.",fun);

  // zjisteni dimenze vysledneho pole
  new_dim = 0;
  dmax = A1.Dim()-1;
  for (d=1; d<=dmax; d++) {
    new_len[new_dim] = A1.Length(d);
    new_dim++;
  }

  dmax = A2.Dim();
  for (d=2; d<=dmax; d++) {
    new_len[new_dim] = A2.Length(d);
    new_dim++;
  }

  // alokace vysledne matice
  switch (new_dim) {
    case 1:
      target.Allocate(new_len[0]);
      break;
    case 2:
      target.Allocate(new_len[0],new_len[1]);
      break;
    case 3:
      target.Allocate(new_len[0],new_len[1],new_len[2]);
      break;
    case 4:
      target.Allocate(new_len[0],new_len[1],new_len[2],new_len[3]);
  }

  // nasobeni
  if (A1.Dim()==2) {
    rmax1 = A1.Length(1);
    cmax1 = A1.Length(2);
    cmax2 = A2.Length(2);

    for (row=1; row<=rmax1; row++) {
      for (col=1; col<=cmax2; col++) {
        sum = 0;
        for (idx=1; idx<=cmax1; idx++)
          sum += A1(row,idx)*A2(idx,col);
        target(row,col) = sum;
      }
    }
  } else
    ErrMessage("Multiplication of these dimension has not been provided yet.",fun);

  return target;
}


// operator deleni cislem
template<class T, int rank>
TArray<T,rank> &TArray<T,rank>::operator/(double value)
{
//  char *fun="TArray<T,rank>::operator/";
  long int i;

  for (i=0; i<items; i++)
    data[i] /= value;
  return *this;
}


/*---------------------------------------------------------------------------------------------------------------------
  class TArray - OSTATNI METODY
---------------------------------------------------------------------------------------------------------------------*/

// nastaveni parametru pro zvetseni struktury zapisem mimo rozsah
template<class T, int rank>
void TArray<T,rank>::EnlargeStep(int step1, ...)
{
  va_list args;
  int i;

  auto_enlarge = true;
  estep[0] = step1;
  va_start(args,step1);
  for (i=1; i<rank; i++)
    estep[i] = va_arg(args,int);
  va_end(args);
}


// vyplneni tenzoru konstantni hodnotou
template<class T, int rank>
void TArray<T,rank>::Fill(double value)
{
  long int i;

  for (i=0; i<items; i++)
    data[i] = value;
}


// vyplneni tenzoru konstantni hodnotou
template<class T, int rank>
void TArray<T,rank>::Resize(int i, int j, int k, int l)
{
  T *new_data;
  long int idx_target,idx_source;
  int ilen,jlen,klen,llen,sT;

  if (!auto_enlarge) {
    ErrMessage("Cannot resize the fixed-size structure (use function EnlargeStep)",
               "TArray<T,rank>::Resize");
    return;
  }

  // vypocet nove velikost struktury
  items = 1;
  switch (dim) {
    case 4:
      if (dim>3 && l>len[3] && estep[3]>0) {
        llen = len[3]+sup((l-len[3])/double(estep[3]))*estep[3];
        items *= llen;
      } else {
        items *= len[3];
        llen = len[3];
      }

    case 3:
      if (dim>2 && k>len[2] && estep[2]>0) {
        klen = len[2]+sup((k-len[2])/double(estep[2]))*estep[2];
        items *= klen;
      } else {
        items *= len[2];
        klen = len[2];
      }

    case 2:
      if (j>len[1] && estep[1]>0) {
        jlen = len[1]+sup((j-len[1])/double(estep[1]))*estep[1];
        items *= jlen;
      } else {
        items *= len[1];
        jlen = len[1];
      }

    case 1:
      if (i>len[0] && estep[0]>0) {
        ilen = len[0]+sup((i-len[0])/double(estep[0]))*estep[0];
        items *= ilen;
      } else {
        items *= len[0];
        ilen = len[0];
      }
  }

  // zvetseni struktury
  sT = sizeof(T);
  new_data = (T *) malloc(items*sT);
  if (new_data==NULL)
    ErrMessage("Out of memory during dynamic allocation","TArray<T,rank>::Resize");

  memset(new_data,0,items*sT);

  switch (dim) {
    case 1:
      memcpy(new_data,data,len[0]*sT);
      len[0] = ilen;
      break;

    case 2:
      idx_source = 0;
      for (i=1; i<=len[0]; i++) {
        for (j=1; j<=len[1]; j++) {
          idx_target = jlen*(i-1)+(j-1);
          new_data[idx_target] = data[idx_source];
          idx_source++;
        }
      }
      len[0] = ilen;
      len[1] = jlen;
      break;

    case 3:
      idx_source = 0;
      for (k=1; k<=len[2]; k++) {
        for (i=1; i<=len[0]; i++) {
          for (j=1; j<=len[1]; j++) {
            idx_target = ilen*jlen*(k-1)+jlen*(i-1)+j-1;
            new_data[idx_target] = data[idx_source];
            idx_source++;
          }
        }
      }
      len[0] = ilen;
      len[1] = jlen;
      len[2] = klen;
      break;

    case 4:
      idx_source = 0;
      for (l=1; l<=len[3]; l++) {
        for (k=1; k<=len[2]; k++) {
          for (i=1; i<=len[0]; i++) {
            for (j=1; j<=len[1]; j++) {
              idx_target = ilen*jlen*klen*(l-1)+ilen*jlen*(k-1)+jlen*(i-1)+j-1;
              new_data[idx_target] = data[idx_source];
              idx_source++;
            }
          }
        }
      }
      len[0] = ilen;
      len[1] = jlen;
      len[2] = klen;
      len[3] = llen;      
      break;
  }

  free(data);
  data = new_data;
}


// prevod vektoru na matici
template<class T, int rank>
DMatrix &TArray<T,rank>::ToMatrix()
{
  static DMatrix mat;

  mat.items = items;
  mat.dim = 2;
  mat.len[1] = len[0];
  mat.len[0] = 1;

  memcpy(mat.estep,estep,4*sizeof(int));
  mat.auto_enlarge = auto_enlarge;

  mat.data = (T *) malloc(items*sizeof(T));
  memcpy(mat.data,data,items*sizeof(T));

  return mat;
}


// prevod matice na vektor - pouze radkova matice (1,n)
template<class T, int rank>
DVector &TArray<T,rank>::ToVector()
{
  static DVector vect;

  vect.items = items;
  vect.dim = 1;
  vect.len[0] = len[1];
  vect.len[1] = 0;

  memcpy(vect.estep,estep,4*sizeof(int));
  vect.auto_enlarge = auto_enlarge;

  vect.data = (T *) malloc(items*sizeof(T));
  memcpy(vect.data,data,items*sizeof(T));

  return vect;
}

#endif
